Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Specialized
Imports System.ComponentModel
Imports System.Data.Linq
Imports System.Linq
Imports System.Linq.Expressions
Imports System.Reflection

Public Module DataContextExtensions

    <System.Runtime.CompilerServices.Extension()> _
    Function [Select](Of TEntity As Class)(ByVal dataContext As DataContext) As IQueryable(Of TEntity)
        Return dataContext.GetTable(Of TEntity)()
    End Function

    <System.Runtime.CompilerServices.Extension()> _
    Function [Get](Of TEntity As {Class, New})(ByVal dataContext As DataContext, ByVal id As Object) As TEntity
        Return [Get](Of TEntity)(dataContext, id, "id")
    End Function

    <System.Runtime.CompilerServices.Extension()> _
    Function [Get](Of TEntity As {Class, New})(ByVal dataContext As DataContext, ByVal id As Object, ByVal primaryKeyName As String) As TEntity
        If id Is Nothing Then
            Return New TEntity()
        End If

        Dim table = dataContext.GetTable(Of TEntity)()
        Return table.Single(DynamicGet(Of TEntity)(primaryKeyName, id))
    End Function

    <System.Runtime.CompilerServices.Extension()> _
    Function Save(Of TEntity As {Class, New})(ByVal dataContext As DataContext, ByVal formParams As NameValueCollection, ByVal id As Integer?) As TEntity
        Return Save(Of TEntity)(dataContext, formParams, id, "id")
    End Function


    <System.Runtime.CompilerServices.Extension()> _
    Function Save(Of TEntity As {Class, New})(ByVal dataContext As DataContext, ByVal formParams As NameValueCollection, ByVal id As Integer?, ByVal primaryKeyName As String) As TEntity
        If id Is Nothing OrElse id < 1 Then
            Return Insert(Of TEntity)(dataContext, formParams)
        Else
            Return Update(Of TEntity)(dataContext, formParams, id.Value)
        End If
    End Function


    <System.Runtime.CompilerServices.Extension()> _
    Function Insert(Of TEntity As {Class, New})(ByVal dataContext As DataContext, ByVal formParams As NameValueCollection) As TEntity
        ' Create entity
        Dim entityToInsert = CreateEntityFromForm(Of TEntity)(dataContext, formParams)

        ' Submit new entity
        Dim table = dataContext.GetTable(Of TEntity)()
        table.InsertOnSubmit(entityToInsert)
        dataContext.SubmitChanges()

        ' Return entity
        Return entityToInsert
    End Function

    <System.Runtime.CompilerServices.Extension()> _
    Function Update(Of TEntity As {Class, New})(ByVal dataContext As DataContext, ByVal formParams As NameValueCollection, ByVal id As Object) As TEntity
        Return Update(Of TEntity)(dataContext, formParams, id, "id")
    End Function

    <System.Runtime.CompilerServices.Extension()> _
    Function Update(Of TEntity As {Class, New})(ByVal dataContext As DataContext, ByVal formParams As NameValueCollection, ByVal id As Object, ByVal primaryKeyName As String) As TEntity
        ' Get matching entity
        Dim entityToUpdate = dataContext.Get(Of TEntity)(id, primaryKeyName)

        ' Update entity
        UpdateEntityFromForm(dataContext, entityToUpdate, formParams)

        ' Save changes
        dataContext.SubmitChanges()
        Return entityToUpdate
    End Function

    <System.Runtime.CompilerServices.Extension()> _
    Sub Delete(Of TEntity As {Class, New})(ByVal dataContext As DataContext, ByVal id As Object)
        Delete(Of TEntity)(dataContext, id, "id")
    End Sub


    <System.Runtime.CompilerServices.Extension()> _
    Sub Delete(Of TEntity As {Class, New})(ByVal dataContext As DataContext, ByVal id As Object, ByVal primaryKeyName As String)
        ' Get matching entity
        Dim entityToDelete = [Get](Of TEntity)(dataContext, id, primaryKeyName)

        ' Delete it
        Dim table = dataContext.GetTable(Of TEntity)()
        table.DeleteOnSubmit(entityToDelete)
        dataContext.SubmitChanges()
    End Sub

    <System.Runtime.CompilerServices.Extension()> _
    Function CreateEntityFromForm(Of TEntity As {Class, New})(ByVal dataContext As DataContext, ByVal formParams As NameValueCollection) As TEntity
        Dim entity = New TEntity()
        For Each propName In formParams.AllKeys
            Dim prop = GetCaseInsensitivePropertyInfo(Of TEntity)(propName)
            If prop IsNot Nothing Then
                Dim converter = TypeDescriptor.GetConverter(prop.PropertyType)
                Dim propValue = converter.ConvertFromInvariantString(formParams(prop.Name))
                prop.SetValue(entity, propValue, Nothing)
            End If
        Next propName
        Return entity
    End Function


    <System.Runtime.CompilerServices.Extension()> _
    Function UpdateEntityFromForm(Of TEntity As {Class, New})(ByVal dataContext As DataContext, ByVal entityToUpdate As TEntity, ByVal formParams As NameValueCollection) As TEntity
        For Each propName In formParams.AllKeys
            Dim prop = GetCaseInsensitivePropertyInfo(Of TEntity)(propName)
            If prop IsNot Nothing Then
                Dim converter = TypeDescriptor.GetConverter(prop.PropertyType)
                Dim propValue = converter.ConvertFromInvariantString(formParams(prop.Name))
                prop.SetValue(entityToUpdate, propValue, Nothing)
            End If
        Next propName
        Return entityToUpdate
    End Function



    Private Function DynamicGet(Of TEntity)(ByVal primaryKeyName As String, ByVal id As Object) As Expression(Of Func(Of TEntity, Boolean))
        Dim e = Expression.Parameter(GetType(TEntity), "e")
        Dim propInfo = GetCaseInsensitivePropertyInfo(Of TEntity)(primaryKeyName)
        Dim m = Expression.MakeMemberAccess(e, propInfo)
        Dim c = Expression.Constant(id)
        Dim b = Expression.Equal(m, c)
        Dim lambda = Expression.Lambda(Of Func(Of TEntity, Boolean))(b, e)
        Return lambda
    End Function

    Private Function GetCaseInsensitivePropertyInfo(Of TEntity)(ByVal propertyName As String) As PropertyInfo
        Return GetType(TEntity).GetProperty(propertyName, BindingFlags.GetProperty Or BindingFlags.Instance Or BindingFlags.Public Or BindingFlags.IgnoreCase)
    End Function


End Module
